﻿using System.Web;
using System.Text;
using System.Security.Cryptography;
using System.IO;
using System.Net;
using System;
using System.Collections.Generic;
using System.Xml;

namespace Com.Alipay
{
    /// <summary>
    /// 类名：Core
    /// 功能：支付宝接口公用函数类
    /// 详细：该类是请求、通知返回两个文件所调用的公用函数核心处理文件，不需要修改
    /// 版本：3.4
    /// 修改日期：2016-03-08
    /// 说明：
    /// 以下代码只是为了方便商户测试而提供的样例代码，商户可以根据自己网站的需要，按照技术文档编写,并非一定要使用该代码。
    /// 该代码仅供学习和研究支付宝接口使用，只是提供一个参考。
    /// File name:AlipayCore
    /// Function:Alipay public constructor
    /// Detail:This is the core public constructor processing file of AlipaySubmit and AlipayNotify.No need to modify.
    /// version:3.3
    /// modify date:2012-08-14
    /// instructions:
    /// This code below is a sample demo for merchants to do test.Merchants can refer to the integration documents and write your own code to fit your website.Not necessarily to use this code.  
    /// Alipay provide this code for you to study and research on Alipay interface, just for your reference.
    /// </summary>
    public class Core
    {

        public Core()
        {
        }

        /// <summary>
        /// 除去数组中的空值和签名参数并以字母a到z的顺序排序
		/// Remove the blank ,sign and sign_type,and rearrange alphabetical from a to z
        /// </summary>
        /// <param name="dicArrayPre">过滤前的参数组the array to be filter</param>
        /// <returns>过滤后的参数组Array filterred</returns>
        public static Dictionary<string, string> FilterPara(SortedDictionary<string, string> dicArrayPre)
        {
            Dictionary<string, string> dicArray = new Dictionary<string, string>();
            foreach (KeyValuePair<string, string> temp in dicArrayPre)
            {
                if (temp.Key.ToLower() != "sign" && temp.Key.ToLower() != "sign_type1" && temp.Value != "" && temp.Value != null)
                {
                    dicArray.Add(temp.Key, temp.Value);
                }
            }

            return dicArray;
        }

        /// <summary>
        /// 把数组所有元素，按照“参数=参数值”的模式用“&”字符拼接成字符串
		/// connect all parameters with & like "parameter name=value"
        /// </summary>
        /// <param name="sArray">需要拼接的数组parameters need to be connected </param>
        /// <returns>拼接完成以后的字符串pre-sign string</returns>
        public static string CreateLinkString(Dictionary<string, string> dicArray)
        {
            StringBuilder prestr = new StringBuilder();
            foreach (KeyValuePair<string, string> temp in dicArray)
            {
                prestr.Append(temp.Key + "=" + temp.Value + "&");
            }

            //去掉最後一個&字符remove the last &
            int nLen = prestr.Length;
            prestr.Remove(nLen-1,1);

            return prestr.ToString();
        }

        /// <summary>
        /// 把数组所有元素，按照“参数=参数值”的模式用“&”字符拼接成字符串，并对参数值做urlencode
		/// connect all parameters with & like "parameter name=value",and do urlencode to the value of parameters
        /// </summary>
        /// <param name="sArray">需要拼接的数组parameters need to be connected</param>
        /// <param name="code">字符编码charset</param>
        /// <returns>拼接完成以后的字符串String connected</returns>
        public static string CreateLinkStringUrlencode(Dictionary<string, string> dicArray, Encoding code)
        {
            StringBuilder prestr = new StringBuilder();
            foreach (KeyValuePair<string, string> temp in dicArray)
            {
                prestr.Append(temp.Key + "=" + HttpUtility.UrlEncode(temp.Value, code) + "&");
            }

            //去掉最後一個&字符
			//remove the last &
            int nLen = prestr.Length;
            prestr.Remove(nLen - 1, 1);

            return prestr.ToString();
        }

        /// <summary>
        /// 写日志，方便测试（看网站需求，也可以改成把记录存入数据库）
	 /// Write the log for your convienence in testing(You can also load these record into database ,it depends on your requirement)
        /// <param name="sWord">要写入日志里的文本内容The string to be written in your log</param>
        public static void LogResult(string sWord)
        {
            string strPath = Config.log_path + "\\" + "alipay_log_" + DateTime.Now.ToString("yyyyMMddHHmmss") + ".txt";
            StreamWriter fs = new StreamWriter(strPath, false, System.Text.Encoding.Default);
            fs.Write(sWord);
            fs.Close();
        }

        /// <summary>
        /// 获取文件的md5摘要generate md5 summary of file
        /// </summary>
        /// <param name="sFile">文件流 file stream</param>
        /// <returns>MD5摘要结果summary result of MD5</returns>
        public static string GetAbstractToMD5(Stream sFile)
        {
            MD5 md5 = new MD5CryptoServiceProvider();
            byte[] result = md5.ComputeHash(sFile);
            StringBuilder sb = new StringBuilder(32);
            for (int i = 0; i < result.Length; i++)
            {
                sb.Append(result[i].ToString("x").PadLeft(2, '0'));
            }
            return sb.ToString();
        }

        /// <summary>
        /// 获取文件的md5摘要
		/// 获取文件的md5摘要generate md5 summary of file
        /// </summary>
        /// <param name="dataFile">文件流file stream</param>
        /// <returns>MD5摘要结果summary result of MD5</returns>
        public static string GetAbstractToMD5(byte[] dataFile)
        {
            MD5 md5 = new MD5CryptoServiceProvider();
            byte[] result = md5.ComputeHash(dataFile);
            StringBuilder sb = new StringBuilder(32);
            for (int i = 0; i < result.Length; i++)
            {
                sb.Append(result[i].ToString("x").PadLeft(2, '0'));
            }
            return sb.ToString();
        }
    }
}